"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !exports.hasOwnProperty(p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAuth = exports.FirebaseAuthentication = void 0;
__exportStar(require("./@types"), exports);
class FirebaseAuthentication {
    constructor(email, key, projectId) {
        this.email = email;
        this.key = key;
        this.projectId = projectId;
    }
    static getAuth(email, key, projectId) {
        return new FirebaseAuthentication(email, key, projectId);
    }
    getUserByUid(uid) {
        const response = this.requestWithAuth('/accounts:lookup', 'post', {
            localId: [uid],
        });
        return this.pickUser(response.users);
    }
    getUserByLocalId(localId) {
        // Just alias
        return this.getUserByUid(localId);
    }
    getUserByEmail(email) {
        const response = this.requestWithAuth('/accounts:lookup', 'post', {
            email: [email],
        });
        return this.pickUser(response.users);
    }
    getUserByPhoneNumber(phoneNumber) {
        const response = this.requestWithAuth('/accounts:lookup', 'post', {
            phoneNumber: [phoneNumber],
        });
        console.log(response.users[0].providerUserInfo);
        return this.pickUser(response.users);
    }
    getUsers(maxResults = 1000) {
        const response = this.requestWithAuth('/accounts:batchGet', 'get', { maxResults });
        return response.users || [];
    }
    createUser(params) {
        return this.requestWithAuth('/accounts', 'post', params);
    }
    createUsers(users) {
        return this.requestWithAuth('/accounts:batchCreate', 'post', { users });
    }
    updateUser(params) {
        return this.requestWithAuth('/accounts:update', 'post', params);
    }
    deleteUser(localId) {
        return this.requestWithAuth('/accounts:delete', 'post', { localId });
    }
    deleteUsers(localIds, force = false) {
        if (localIds.length === 0) {
            return null;
        }
        return this.requestWithAuth('/accounts:batchDelete', 'post', { localIds, force });
    }
    pickUser(users) {
        if (!users || users.length === 0) {
            return null;
        }
        else {
            return users[0];
        }
    }
    requestWithAuth(path, method, data) {
        const options = {
            method,
            contentType: 'application/json',
            headers: {
                Authorization: `Bearer ${this.accessToken}`,
            },
        };
        if (data) {
            if (method === 'get') {
                path = `${path}?${Object.entries(data).map(([k, v]) => `${k}=${v}`).join('&')}`;
            }
            else {
                options.payload = JSON.stringify(data);
            }
        }
        const url = `https://identitytoolkit.googleapis.com/v1/projects/${this.projectId}${path}`;
        const response = UrlFetchApp.fetch(url, options);
        return JSON.parse(response.getContentText());
    }
    get accessToken() {
        // This access token will expire after 1 hour later, but GAS will terminate 30 min for Google limitation.
        // Therefore, we don't care refresh it.
        if (!this._accessToken) {
            this._accessToken = this.getAccessToken();
        }
        return this._accessToken;
    }
    getAccessToken() {
        const GOOGLE_AUTH_TOKEN_HOST = 'accounts.google.com';
        const GOOGLE_AUTH_TOKEN_PATH = '/o/oauth2/token';
        const token = this.createAuthJwt();
        const postData = 'grant_type=urn%3Aietf%3Aparams%3Aoauth%3A' +
            'grant-type%3Ajwt-bearer&assertion=' +
            token;
        const url = `https://${GOOGLE_AUTH_TOKEN_HOST}${GOOGLE_AUTH_TOKEN_PATH}`;
        const response = UrlFetchApp.fetch(url, {
            method: 'post',
            contentType: 'application/x-www-form-urlencoded',
            payload: postData,
        });
        const { access_token } = JSON.parse(response.getContentText());
        return access_token;
    }
    createAuthJwt() {
        const GOOGLE_TOKEN_AUDIENCE = 'https://accounts.google.com/o/oauth2/token';
        const ONE_HOUR_IN_SECONDS = 60 * 60;
        const scope = [
            'https://www.googleapis.com/auth/cloud-platform',
            'https://www.googleapis.com/auth/firebase.database',
            'https://www.googleapis.com/auth/firebase.messaging',
            'https://www.googleapis.com/auth/identitytoolkit',
            'https://www.googleapis.com/auth/userinfo.email',
        ].join(' ');
        const jwtHeader = {
            alg: 'RS256',
        };
        const seconds = ~~(new Date().getTime() / 1000);
        const jwtPayload = {
            iss: this.email,
            scope,
            aud: GOOGLE_TOKEN_AUDIENCE,
            exp: seconds + ONE_HOUR_IN_SECONDS,
            iat: seconds,
        };
        const jwtHeaderBase64 = Utilities.base64EncodeWebSafe(JSON.stringify(jwtHeader));
        const jwtClaimBase64 = Utilities.base64EncodeWebSafe(JSON.stringify(jwtPayload));
        const signatureInput = `${jwtHeaderBase64}.${jwtClaimBase64}`;
        const signature = Utilities.computeRsaSha256Signature(signatureInput, this.key);
        return `${signatureInput}.${Utilities.base64EncodeWebSafe(signature)}`;
    }
}
exports.FirebaseAuthentication = FirebaseAuthentication;
function getAuth(email, key, projectId) {
    return FirebaseAuthentication.getAuth(email, key, projectId);
}
exports.getAuth = getAuth;
